@@ -86,9 +86,9 @@ import org.jline.terminal.Attributes;
 import org.jline.terminal.Terminal;
 import org.jline.utils.AttributedString;
 import org.jline.utils.AttributedStringBuilder;
-import org.jline.utils.AttributedStyle;
 import org.jline.utils.InfoCmp.Capability;
 import org.jline.utils.OSUtils;
+import org.jline.utils.StyleResolver;
 
 /**
  * Posix-like utilities.
@@ -104,7 +104,8 @@ public class Posix {
         // TTop function is new in JLine 3.2
         String[] func;
         try {
-            Class cl = TTop.class;
+            @SuppressWarnings("unused")
+            Class<?> cl = TTop.class;
             func = new String[] {
                     "cat", "echo", "grep", "sort", "sleep", "cd", "pwd", "ls",
                     "less", "watch", "nano", "tmux",
@@ -154,6 +155,7 @@ public class Posix {
         }
     }
 
+    @SuppressWarnings("serial")
     protected static class HelpException extends Exception {
         public HelpException(String message) {
             super(message);
@@ -388,9 +390,6 @@ public class Posix {
         boolean displayWords = opt.isSet("words");
         boolean displayChars = opt.isSet("chars");
         boolean displayBytes = opt.isSet("bytes");
-        if (displayChars) {
-            displayBytes = false;
-        }
         if (!displayLines && !displayWords && !displayChars && !displayBytes) {
             displayLines = true;
             displayWords = true;
@@ -398,18 +397,36 @@ public class Posix {
         }
         String format = "";
         if (displayLines) {
-            format += "%1$8d";
+            if (!displayBytes && !displayChars && !displayWords) {
+                format = "%1$d";
+            } else {
+                format += "%1$8d";
+            }
         }
         if (displayWords) {
-            format += "%2$8d";
+            if (!displayLines && !displayBytes && !displayChars) {
+                format = "%2$d";
+            } else {
+                format += "%2$8d";
+            }
         }
         if (displayChars) {
-            format += "%3$8d";
+            if (!displayLines && !displayBytes && !displayWords) {
+                format = "%3$d";
+            } else {
+                format += "%3$8d";
+            }
         }
         if (displayBytes) {
-            format += "%4$8d";
+            if (!displayLines && !displayChars && !displayWords) {
+                format = "%4$d";
+            } else {
+                format += "%4$8d";
+            }
+        }
+        if (sources.size() > 1 || (sources.size() == 1 && sources.get(0).getName() != null)) {
+            format += "  %5$8s";
         }
-        format += "  %5s";
         int totalLines = 0;
         int totalBytes = 0;
         int totalChars = 0;
@@ -770,6 +787,7 @@ public class Posix {
                 "Usage: clear [OPTIONS]",
                 "  -? --help                    Show help",
         };
+        @SuppressWarnings("unused")
         Options opt = parseOptions(session, usage, argv);
         if (process.isTty(1)) {
             Shell.getTerminal(session).puts(Capability.clear_screen);
@@ -998,7 +1016,7 @@ public class Posix {
         List<String> sortFields = opt.getList("key");
 
         char sep = (separator == null || separator.length() == 0) ? '\0' : separator.charAt(0);
-        Collections.sort(lines, new SortComparator(caseInsensitive, reverse, ignoreBlanks, numeric, sep, sortFields));
+        lines.sort(new SortComparator(caseInsensitive, reverse, ignoreBlanks, numeric, sep, sortFields));
         String last = null;
         for (String s : lines) {
             if (!unique || last == null || !s.equals(last)) {
@@ -1158,13 +1176,9 @@ public class Posix {
                     type = "";
                     suffix = "";
                 }
-                String col = colors.get(type);
                 boolean addSuffix = opt.isSet("F");
-                if (col != null && !col.isEmpty()) {
-                    return "\033[" + col + "m" + path.toString() + "\033[m" + (addSuffix ? suffix : "") + link;
-                } else {
-                    return path.toString() + (addSuffix ? suffix : "") + link;
-                }
+                return applyStyle(path.toString(), colors, type)
+                        + (addSuffix ? suffix : "") + link;
             }
 
             String longDisplay() {
@@ -1258,7 +1272,7 @@ public class Posix {
                     try {
                         Map<String, Object> ta = Files.readAttributes(path, view + ":*",
                                 getLinkOptions(opt.isSet("L")));
-                        ta.entrySet().forEach(e -> attrs.putIfAbsent(e.getKey(), e.getValue()));
+                        ta.forEach(attrs::putIfAbsent);
                     } catch (IOException e) {
                         // Ignore
                     }
@@ -1278,7 +1292,8 @@ public class Posix {
             opt.args().forEach(s -> expanded.add(currentDir.resolve(s)));
         }
         boolean listAll = opt.isSet("a");
-        Predicate<Path> filter = p -> listAll || !p.getFileName().toString().startsWith(".");
+		Predicate<Path> filter = p -> listAll || p.getFileName().toString().equals(".")
+				|| p.getFileName().toString().equals("..") || !p.getFileName().toString().startsWith(".");
         List<PathEntry> all = expanded.stream()
                 .filter(filter)
                 .map(p -> new PathEntry(p, currentDir))
@@ -1337,7 +1352,7 @@ public class Posix {
             if (expanded.size() > 1) {
                 out.println(currentDir.relativize(path).toString() + ":");
             }
-            display.accept(Stream.concat(Arrays.asList(".", "..").stream().map(path::resolve), Files.list(path))
+            display.accept(Stream.concat(Stream.of(".", "..").map(path::resolve), Files.list(path))
                             .filter(filter)
                             .map(p -> new PathEntry(p, path))
                             .sorted()
@@ -2058,26 +2073,36 @@ public class Posix {
     public static Map<String, String> getColorMap(CommandSession session, String name, String def) {
         Object obj = session.get(name + "_COLORS");
         String str = obj != null ? obj.toString() : null;
-        if (str == null || !str.matches("[a-z]{2}=[0-9]*(;[0-9]+)*(:[a-z]{2}=[0-9]*(;[0-9]+)*)*")) {
+        if (str == null) {
             str = def;
         }
-        return Arrays.stream(str.split(":"))
+        String sep = str.matches("[a-z]{2}=[0-9]*(;[0-9]+)*(:[a-z]{2}=[0-9]*(;[0-9]+)*)*") ? ":" : " ";
+        return Arrays.stream(str.split(sep))
                 .collect(Collectors.toMap(s -> s.substring(0, s.indexOf('=')),
                                           s -> s.substring(s.indexOf('=') + 1)));
     }
 
-    private void applyStyle(AttributedStringBuilder sb, Map<String, String> colors, String... types) {
-        String col = null;
+    static String applyStyle(String text, Map<String, String> colors, String... types) {
+        String t = null;
         for (String type : types) {
-            col = colors.get(type);
-            if (col != null) {
+            if (colors.get(type) != null) {
+                t = type;
                 break;
             }
         }
-        sb.style(AttributedStyle.DEFAULT);
-        if (col != null && !col.isEmpty()) {
-            sb.appendAnsi("\033[" + col + "m");
+        return new AttributedString(text, new StyleResolver(colors::get).resolve("." + t))
+                .toAnsi();
+    }
+
+    static void applyStyle(AttributedStringBuilder sb, Map<String, String> colors, String... types) {
+        String t = null;
+        for (String type : types) {
+            if (colors.get(type) != null) {
+                t = type;
+                break;
+            }
         }
+        sb.style(new StyleResolver(colors::get).resolve("." + t));
     }
 
     private static class StdInSource implements Source {
@@ -2094,7 +2119,7 @@ public class Posix {
         }
 
         @Override
-        public InputStream read() throws IOException {
+        public InputStream read() {
             return process.in();
         }
     }
